From 3ba23aea28796b801832baf1aa5e1092cff88e6e Mon Sep 17 00:00:00 2001
From: Prateek Sharma <ps.prateek.sharma143@gmail.com>
Date: Sat, 20 Apr 2024 02:06:17 +0530
Subject: [PATCH] Fix treesitter-font-lock-settings for built-in functions and
 attributes.

* etc/NEWS: Announce the effect of the changes for the treesitter-font-lock-settings for built-in functions and attributes
* lisp/progmodes/python.el (python--treesit-settings): Change the treesitter query to fetch the correct type of node for built-in functions and attributes and highlight them with corresponding font-lock face.
---
 etc/NEWS                 |  9 +++++++++
 lisp/progmodes/python.el | 13 ++++++-------
 2 files changed, 15 insertions(+), 7 deletions(-)

diff --git a/etc/NEWS b/etc/NEWS
index 8ad1e78ca60..e5cd70b6106 100644
--- a/etc/NEWS
+++ b/etc/NEWS
@@ -1319,6 +1319,15 @@ instead of:
 This allows the user to specify command line arguments to the non
 interactive Python interpreter specified by 'python-interpreter'.
 
+*** Treesitter syntax highlighting for built-in functions fixed
+The fix highlights only the correct built-in function calls and
+attributes with font-lock-builtin-face. When some class defines
+a function with the same name as a built-in function, earlier it was
+being highlighted with the same font-lock-builtin-face, which was
+incorrect. Now, normal user defined functions will be highlighted
+with font-lock-function-call-face face and built-in functions and
+attributes will be highlighted with font-lock-builtin-face.
+
 ** Scheme mode
 Scheme mode now handles regular expression literal '#/regexp/' that is
 available in some Scheme implementations.
diff --git a/lisp/progmodes/python.el b/lisp/progmodes/python.el
index 85279d3e84b..5c67d0765eb 100644
--- a/lisp/progmodes/python.el
+++ b/lisp/progmodes/python.el
@@ -1206,13 +1206,12 @@ fontified."
 
    :feature 'builtin
    :language 'python
-   `(((identifier) @font-lock-builtin-face
-      (:match ,(rx-to-string
-                `(seq bol
-                      (or ,@python--treesit-builtins
-                          ,@python--treesit-special-attributes)
-                      eol))
-              @font-lock-builtin-face)))
+   `((call function: (identifier) @font-lock-builtin-face
+           (:match ,(rx-to-string `(seq bol (or ,@python--treesit-builtins) eol))
+                   @font-lock-builtin-face))
+     (attribute attribute: (identifier) @font-lock-builtin-face
+                (:match ,(rx-to-string `(seq bol (or ,@python--treesit-special-attributes) eol))
+                        @font-lock-builtin-face)))
 
    :feature 'decorator
    :language 'python
-- 
2.39.3 (Apple Git-146)

